import { BaseChat, BaseChatOptions } from '../core/chat.ts';
import { numTokensFromString } from '../utils/token.ts';

export type KnowledgeOptions<T = Record<string, string>> = BaseChatOptions<
  {
    embeddingModel: string;
    splitSize?: number; // 分块大小 默认 2000
    splitOverlap?: number; // 分块重叠 默认 200
    batchSize?: number; // 批量大小 默认 4, 4*2000=8000
  } & T
>;
/**
 * 知识库构建
 * 1. Embedding generate
 * 2. retriever
 * 3. reranker
 */
export class KnowledgeBase extends BaseChat {
  embeddingModel: string;
  splitSize: number;
  splitOverlap: number;
  batchSize: number;
  constructor(options: KnowledgeOptions) {
    super(options);
    this.embeddingModel = options.embeddingModel;
    this.splitSize = options.splitSize || 2000;
    this.splitOverlap = options.splitOverlap || 200;
    this.prompt_tokens = 0;
    this.total_tokens = 0;
    this.batchSize = options.batchSize || 4;
  }
  /**
   * 生成embedding 内部
   * @param text
   * @returns
   */
  async generateEmbeddingCore(text: string | string[]) {
    const res = await this.openai.embeddings.create({
      model: this.embeddingModel,
      input: text,
      encoding_format: 'float',
    });
    this.prompt_tokens += res.usage.prompt_tokens;
    this.total_tokens += res.usage.total_tokens;
    return res;
  }
  async generateEmbeddingBatchCore(text: string[]) {
    const res = await this.openai.embeddings.create({
      model: this.embeddingModel,
      input: text,
      encoding_format: 'float',
    });
    this.prompt_tokens += res.usage.prompt_tokens;
    this.total_tokens += res.usage.total_tokens;
    return res.data.map((item) => item.embedding);
  }
  /**
   * 生成embedding
   * @param text
   * @returns
   */
  async generateEmbedding(text: string | string[]) {
    if (Array.isArray(text)) {
      // size token 不能超过 8192
      const allSize = text.reduce((acc, item) => acc + numTokensFromString(item), 0);
      if (allSize > 8192) {
        throw new Error('text size 不能超过 8192');
      }
    }
    const res = await this.generateEmbeddingCore(text);
    if (Array.isArray(text)) {
      return res.data.map((item) => item.embedding);
    }
    return [res.data[0].embedding];
  }
  /**
   * 批量生成embedding
   * @param text
   * @returns
   */
  async generateEmbeddingBatch(textArray: string[]) {
    const batchSize = this.batchSize || 4;
    const embeddings: number[][] = [];
    for (let i = 0; i < textArray.length; i += batchSize) {
      const batch = textArray.slice(i, i + batchSize);
      const res = await this.generateEmbeddingBatchCore(batch);
      embeddings.push(...res);
    }
    return embeddings;
  }
  /**
   * 分割长文本, 生成对应的embedding
   * @param text
   * @returns
   */
  async splitLongText(text: string) {
    // 分割文本
    const chunks: string[] = [];
    let startIndex = 0;

    while (startIndex < text.length) {
      // 计算当前chunk的结束位置
      const endIndex = Math.min(startIndex + this.splitSize, text.length);

      // 提取当前chunk
      const chunk = text.substring(startIndex, endIndex);
      chunks.push(chunk);

      // 移动到下一个起始位置，考虑重叠
      startIndex = endIndex - this.splitOverlap;

      // 如果下一个起始位置已经超出或者太接近文本结尾，就结束循环
      if (startIndex >= text.length - this.splitOverlap) {
        break;
      }
    }

    // 为每个chunk生成embedding
    const embeddings = await this.generateEmbeddingBatch(chunks);
    // 返回文本片段和对应的embedding
    return chunks.map((chunk, index) => ({
      text: chunk,
      embedding: embeddings[index],
    }));
  }
}
